@@ -5,14 +5,14 @@
     Pedig <- as.data.frame(Pedig)
     if(PedigAsDataTable){setDF(Pedig)}
     colnames(Pedig)[1:3]<-c("Indiv", "Sire", "Dam")
-    Pedig[,"Indiv"] <- as.character(Pedig[,"Indiv"])
-    Pedig[,"Sire"]  <- as.character(Pedig[,"Sire"])
-    Pedig[,"Dam"]   <- as.character(Pedig[,"Dam"])
-    Pedig[Pedig$Sire %in% c("","0"," "), "Sire"] <- NA
-    Pedig[Pedig$Dam  %in% c("","0"," "),  "Dam"] <- NA
+    Pedig$Indiv <- as.character(Pedig$Indiv)
+    Pedig$Sire  <- as.character(Pedig$Sire)
+    Pedig$Dam   <- as.character(Pedig$Dam)
+    Pedig$Sire[Pedig$Sire %in% c("","0"," ")] <- NA
+    Pedig$Dam[Pedig$Dam  %in% c("","0"," ")] <- NA
     
     if(is.logical(keep)){
-      keep<-Pedig$Indiv[keep]
+      keep <- Pedig$Indiv[keep]
       }
     if(!is.null(keep)){
       keep <- as.character(keep)
@@ -29,7 +29,7 @@
     rownames(Pedig)<-Pedig$Indiv
     
     if("Sex" %in% colnames(Pedig)){
-      Pedig[Pedig$Sex %in% c(""," "), "Sex"] <- NA
+      Pedig$Sex[Pedig$Sex %in% c(""," ")] <- NA
     }else{
       Pedig$Sex<-NA
     }
@@ -38,13 +38,15 @@
       if(!is.character(Pedig$Breed)){
         Pedig$Breed <- as.character(Pedig$Breed)
       }
-      Pedig[Pedig$Breed %in% c(""," "), "Breed"] <- NA
+      Pedig$Breed[Pedig$Breed %in% c(""," ")] <- NA
     }else{
-      if(!is.na(thisBreed)){Pedig$Breed<-thisBreed}
+      if(!is.na(thisBreed)){
+        Pedig$Breed <- thisBreed
+        }
     }
     if("Born" %in% colnames(Pedig) & !is.numeric(Pedig$Born)){
       Pedig$Born <- as.character(Pedig$Born)
-      Pedig[Pedig$Born %in% c(""," "), "Born"] <- NA
+      Pedig$Born[Pedig$Born %in% c(""," ")] <- NA
       Pedig$Born <- as.numeric(Pedig$Born)
     }
     withBreed <- ("Breed" %in% colnames(Pedig))
@@ -59,24 +61,24 @@
       print(Pedig[ord==-1, 2:3])
       cat("\n")
     
-      Pedig[ord==-1,"Sire"]<-NA
-      Pedig[ord==-1,"Dam"]<-NA
-      Pedig[ord==-1,"Breed"]<-"Pedigree Error"
+      Pedig$Sire[ord==-1] <- NA
+      Pedig$Dam[ord==-1]  <- NA
+      Pedig$Breed[ord==-1]<- "Pedigree Error"
     }
  
     ####### Add imaginary ancestors #######
     if(!is.na(lastNative)){
-      rownames(Pedig)<-Pedig[,"Indiv"]
-      ID <-  Pedig[is.na(Pedig[,"Sire"]) & !is.na(Pedig[,"Dam"]), "Indiv"]
-      Pedig[ID, "Sire"]<- paste("S", ID, sep="")
-      ID <-  Pedig[!is.na(Pedig[,"Sire"]) & is.na(Pedig[,"Dam"]), "Indiv"]
-      Pedig[ID, "Dam"]<- paste("D", ID, sep="")
+      rownames(Pedig)<-Pedig$Indiv
+      ID <-  Pedig$Indiv[is.na(Pedig$Sire) & !is.na(Pedig$Dam)]
+      Pedig[ID, "Sire"] <- paste0("S", ID)
+      ID <-  Pedig$Indiv[!is.na(Pedig$Sire) & is.na(Pedig$Dam)]
+      Pedig[ID, "Dam"]  <- paste0("D", ID)
     }
 
     #### Add lines for ancestors, sort pedigree ####
     Pedig <- nadiv::prepPed(Pedig)  
-    Pedig[,"Indiv"] <- as.character(Pedig[,"Indiv"])
-    rownames(Pedig)<- Pedig[,"Indiv"]
+    Pedig$Indiv <- as.character(Pedig$Indiv)
+    rownames(Pedig)<- Pedig$Indiv
     
     Mode <- function(x) {
       ux <- unique(x)
@@ -84,78 +86,77 @@
     }
     
     ### code sexes as 1 (males) and 2 (females) ####
-    if(sum(!is.na(Pedig[,"Sex"]))>0){
-      sexes<-names(table(Pedig[,"Sex"]))
+    if(sum(!is.na(Pedig$Sex))>0){
+      sexes<-names(table(Pedig$Sex))
       if(length(sexes)>2){
         cat("Warning: The sex has more than 2 levels. Please correct it.\n")
       }
       if(length(sexes)==1){sexes<-c(sexes, "dummysex")}
-      Mval <- Mode(Pedig[Pedig[,"Indiv"] %in% Pedig[,"Sire"], "Sex"])
-      Fval <- Mode(Pedig[Pedig[,"Indiv"] %in% Pedig[,"Dam"],  "Sex"])
-      if(is.na(Mval)){Mval<-setdiff(sexes, Fval)}
-      if(is.na(Fval)){Fval<-setdiff(sexes, Mval)}
+      Mval <- Mode(Pedig$Sex[Pedig$Indiv %in% Pedig$Sire])
+      Fval <- Mode(Pedig$Sex[Pedig$Indiv %in% Pedig$Dam])
+      if(is.na(Mval)){Mval <- setdiff(sexes, Fval)}
+      if(is.na(Fval)){Fval <- setdiff(sexes, Mval)}
       if(!is.na(Mval)&!is.na(Fval)&(Mval!=Fval)){
-        MF <- c(1,2)
-        names(MF) <- c(Mval, Fval)
-        Pedig[,"Sex"] <- MF[as.character(Pedig[,"Sex"])]
+        Pedig$Sex <- mapvalues(Pedig$Sex, from=c(Mval, Fval), to=c(1,2))
       }else{
       cat("Meaning of sex labels cannot be determined from pedigree structure.\n")
       }
     }
 
     #### determine sexes from pedigree structure ####
-    wrongMale   <- Pedig[,"Indiv"] %in% Pedig[, "Sire"] & !(Pedig[,"Sex"] %in% c(1, NA))
-    wrongFemale <- Pedig[,"Indiv"] %in% Pedig[, "Dam"]  & !(Pedig[,"Sex"] %in% c(2, NA))
+    wrongMale   <- Pedig$Indiv %in% Pedig$Sire & !(Pedig$Sex %in% c(1, NA))
+    wrongFemale <- Pedig$Indiv %in% Pedig$Dam  & !(Pedig$Sex %in% c(2, NA))
     if(sum(wrongMale)+sum(wrongFemale)>0){
       cat("The sex of the following animals was not compatible with the pedigree, so\n")
       cat("it was modified:\n")
       print(Pedig[wrongMale|wrongFemale, 2:3])
       cat("\n")
     }
-    Pedig[Pedig[,"Indiv"] %in% Pedig[, "Sire"],"Sex"] <- 1
-    Pedig[Pedig[,"Indiv"] %in% Pedig[, "Dam"], "Sex"] <- 2
+    Pedig$Sex[Pedig$Indiv %in% Pedig$Sire] <- 1
+    Pedig$Sex[Pedig$Indiv %in% Pedig$Dam] <- 2
     
     ######  prune Pedigree   #######
     if(!is.null(keep)){
-      Pedig<-nadiv::prunePed(Pedig, phenotyped=keep)
-      Pedig$Indiv<-as.character(Pedig$Indiv)
-      Pedig$Sire<-as.character(Pedig$Sire)
-      Pedig$Dam<-as.character(Pedig$Dam)
+      Pedig <- nadiv::prunePed(Pedig, phenotyped=keep)
+      Pedig$Indiv<- as.character(Pedig$Indiv)
+      Pedig$Sire <- as.character(Pedig$Sire)
+      Pedig$Dam  <- as.character(Pedig$Dam)
     }
     
     ####           Correct wrong breed names              ###
     if(withBreed & !is.na(thisBreed)){
-      hasWrongBreed <- (!is.na(Pedig$Sire) & !is.na(Pedig$Dam) & !(Pedig$Breed %in% c(thisBreed, "Pedigree Error")) & (Pedig[Pedig$Sire, "Breed"] %in% thisBreed) & (Pedig[Pedig$Dam, "Breed"] %in% thisBreed)) 
+      numSire <- match(Pedig$Sire, Pedig$Indiv)
+      numDam <- match(Pedig$Dam, Pedig$Indiv)
+      
+      hasWrongBreed <- (!is.na(Pedig$Sire) & !is.na(Pedig$Dam) & !(Pedig$Breed %in% c(thisBreed, "Pedigree Error")) & (Pedig$Breed[numSire] %in% thisBreed) & (Pedig$Breed[numDam] %in% thisBreed))
       while(any(hasWrongBreed)){
-        cat(paste0("The breed name of the ",sum(hasWrongBreed)," individuals is changed to ",thisBreed))
-        cat(paste0(" because both parents are ", thisBreed,". This includes\n"))
+        cat(paste0("The breed name of ",sum(hasWrongBreed)," individuals is changed to ",thisBreed))
+        cat(paste0(" because both parents are ", thisBreed,".\nThis includes\n"))
         print(head(Pedig[hasWrongBreed,]))
-        Pedig[hasWrongBreed, "Breed"] <- thisBreed
-        hasWrongBreed <- (!is.na(Pedig$Sire) & !is.na(Pedig$Dam) & !(Pedig$Breed %in% c(thisBreed, "Pedigree Error")) & (Pedig[Pedig$Sire, "Breed"] %in% thisBreed) & (Pedig[Pedig$Dam, "Breed"] %in% thisBreed)) 
-      }
+        Pedig$Breed[hasWrongBreed] <- thisBreed
+        hasWrongBreed <- (!is.na(Pedig$Sire) & !is.na(Pedig$Dam) & !(Pedig$Breed %in% c(thisBreed, "Pedigree Error")) & (Pedig$Breed[numSire] %in% thisBreed) & (Pedig$Breed[numDam] %in% thisBreed))
       }
+    }
     
     
     ####             Estimate missing breeds              ###
     ###    Animals with missing breeds are assumed to     ###
     ### be from the same breed as most of their offspring ###
     if(withBreed){
-      ID  <- Pedig[is.na(Pedig[,"Breed"]), "Indiv"]
-      Tab <- Pedig[Pedig[,"Sire"]%in% ID | Pedig[,"Dam"] %in% ID & !is.na(Pedig[,"Breed"]),c("Sire", "Dam", "Breed")] 
-      Tab <- data.frame(ID=c(Tab[,1], Tab[,2]), Breed=c(Tab[,3], Tab[,3]))
-      Pedig[ID, "Breed"] <- tapply(as.character(Tab[,2]),list(Tab[,1]),Mode)[ID]
+      ID  <- Pedig$Indiv[is.na(Pedig$Breed)]
+      Tab <- Pedig[(Pedig$Sire%in% ID | Pedig$Dam %in% ID) & !is.na(Pedig$Breed),c("Sire", "Dam", "Breed")] 
+      Pedig[ID, "Breed"] <- tapply(c(Tab$Breed, Tab$Breed), list(c(Tab$Sire, Tab$Dam)), Mode)[ID]
     }
     
     ####   convert breed name of founders   ###
     ####  born after lastNative to unknown  ###
     if(withBreed & withBorn & !is.na(lastNative)){
-      isFounder <- is.na(Pedig[,"Sire"]) & is.na(Pedig[,"Dam"]) & (Pedig[,"Breed"]==thisBreed | is.na(Pedig[,"Breed"]))
-      notNative <- isFounder & Pedig[,"Born"]>lastNative
-      names(notNative)<-Pedig[,"Indiv"]
-      ID  <- Pedig[is.na(notNative), "Indiv"]
-      Tab <- Pedig[Pedig[,"Sire"]%in% ID | Pedig[,"Dam"] %in% ID & !is.na(Pedig[,"Born"]),c("Sire", "Dam", "Born")] 
-      Tab <- data.frame(ID=c(Tab[,1], Tab[,2]), Born=c(Tab[,3], Tab[,3]))
-      geb <- tapply(Tab[,2],list(Tab[,1]),min)-I
+      isFounder <- is.na(Pedig$Sire) & is.na(Pedig$Dam) & (Pedig$Breed==thisBreed | is.na(Pedig$Breed))
+      notNative <- isFounder & Pedig$Born>lastNative
+      names(notNative)<-Pedig$Indiv
+      ID  <- Pedig$Indiv[is.na(notNative)]
+      Tab <- Pedig[Pedig$Sire%in% ID | Pedig$Dam %in% ID & !is.na(Pedig$Born), c("Sire", "Dam", "Born")] 
+      geb <- tapply(c(Tab$Born, Tab$Born), list(c(Tab$Sire, Tab$Dam)), min) - I
       notNative[ID] <- geb[ID]>lastNative
       Pedig[is.na(notNative) | notNative, "Breed"] <- "unknown"
     }
